"""command-line interface functions for training,
creating learning curves, etc."""
from .prep import prep
from .learncurve import learning_curve
from .predict import predict
from .cli import cli
from .train import train
